{-----------------------------------------------------------------------------
 Unit Name: MainForm
 Author:    Kiriakos Vlahos
 Purpose:   Demonstrates the use of the Scintilla Delphi Wrapper
 History:
-----------------------------------------------------------------------------}

unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdActns, ActnList, ImgList, ActnCtrls, ToolWin, ActnMan,
  ActnMenus, SciLexer, ComCtrls, AppEvnts,  Menus, BandActn,
  XPStyleActnCtrls, OleCtrls, SHDocVw, SciLexerOptionsDlg, SciSearchReplace;

type
  TCEMainForm = class(TForm)
    ActionManager: TActionManager;
    ActionMainMenuBar: TActionMainMenuBar;
    ToolBar: TActionToolBar;
    ImageList: TImageList;
    FileOpen: TFileOpen;
    FileSaveAs: TFileSaveAs;
    FileExit: TFileExit;
    EditCut: TEditCut;
    EditCopy: TEditCopy;
    EditPaste: TEditPaste;
    EditSelectAll: TEditSelectAll;
    EditUndo: TEditUndo;
    EditDelete: TEditDelete;
    HelpContents: THelpContents;
    HelpTopicSearch: THelpTopicSearch;
    HelpOnHelp: THelpOnHelp;
    SearchFind: TAction;
    SearchFindNext: TAction;
    SearchReplace: TAction;
    ActionFileNew: TAction;
    StatusBar: TStatusBar;
    CodeEdit: TScintilla;
    ApplicationEvents: TApplicationEvents;
    ViewStatusBar: TAction;
    ViewToolBar: TAction;
    ToolBarPopup: TPopupMenu;
    Customize1: TMenuItem;
    EditPopup: TPopupMenu;
    Copy1: TMenuItem;
    Cut1: TMenuItem;
    Paste1: TMenuItem;
    Delete1: TMenuItem;
    SelectAll1: TMenuItem;
    Undo1: TMenuItem;
    N1: TMenuItem;
    SaveFile: TAction;
    ReDo: TAction;
    Redo1: TMenuItem;
    N2: TMenuItem;
    Customize: TCustomizeActionBars;
    ZoomIn: TAction;
    ZoomOut: TAction;
    CodeFoldAll: TAction;
    ToggleBookMark: TAction;
    NextBookmark: TAction;
    PreviousBookMark: TAction;
    CPP: TAction;
    Pascal: TAction;
    VBScript: TAction;
    TextFiles: TAction;
    SearchBraces: TAction;
    FileExportToHTML: TAction;
    HTMLSave: TSaveDialog;
    WebBrowser: TWebBrowser;
    FilePrint: TAction;
    FilePrintPreview: TAction;
    PageSetup: TAction;
    HTML: TAction;
    XML: TAction;
    EditorOptions: TAction;
    OptionsDialog: TScintillaOptionsDlg;
    SciSearchReplace: TSciSearchReplace;
    Python: TAction;
    Ruby: TAction;
    procedure ApplicationEventsHint(Sender: TObject);
    procedure ViewToolBarExecute(Sender: TObject);
    procedure ViewStatusBarExecute(Sender: TObject);
    procedure ActionFileNewExecute(Sender: TObject);
    procedure SaveFileExecute(Sender: TObject);
    procedure FileSaveAsAccept(Sender: TObject);
    procedure FileOpenAccept(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ReDoExecute(Sender: TObject);
    procedure ReDoUpdate(Sender: TObject);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ZoomOutExecute(Sender: TObject);
    procedure ZoomInExecute(Sender: TObject);
    procedure CodeFoldAllExecute(Sender: TObject);
    procedure ToggleBookMarkExecute(Sender: TObject);
    procedure NextBookmarkExecute(Sender: TObject);
    procedure PreviousBookMarkExecute(Sender: TObject);
    procedure SyntaxExecute(Sender: TObject);
    procedure SearchFindExecute(Sender: TObject);
    procedure SearchFindNextExecute(Sender: TObject);
    procedure SearchReplaceExecute(Sender: TObject);
    procedure SearchBracesExecute(Sender: TObject);
    procedure FileExportToHTMLExecute(Sender: TObject);
    procedure FilePrintExecute(Sender: TObject);
    procedure FilePrintPreviewExecute(Sender: TObject);
    procedure PageSetupExecute(Sender: TObject);
    procedure EditorOptionsExecute(Sender: TObject);
    procedure SciSearchReplaceTextNotFound(Sender: TObject);
    procedure SyntaxUpdate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FileName : string;
    procedure ContentsToWebBrowser;
  end;

var
  CEMainForm: TCEMainForm;

implementation

Uses
  ActiveX, StrUtils, SciSupport, ScintillaSynLexers;

{$R *.dfm}

var
  MarkerBookmark : integer = 1;

procedure TCEMainForm.FormShow(Sender: TObject);
begin
  CodeEdit.Modified := False;
  ViewToolBar.Checked := ToolBar.Visible;
  SyntaxExecute(Pascal);

  //  Setup bookmarks
  CodeEdit.MarkerSetFore(MarkerBookmark, RGB(00,0,$7F));
  CodeEdit.MarkerSetBack(MarkerBookmark, RGB($80,$FF,$FF));
  CodeEdit.MarkerDefine(MarkerBookmark, SC_MARK_CIRCLE);
end;

procedure TCEMainForm.ApplicationEventsHint(Sender: TObject);
begin
  StatusBar.Panels[0].Text  := GetLongHint(Application.Hint);
end;

procedure TCEMainForm.ViewToolBarExecute(Sender: TObject);
begin
  ToolBar.Visible := (Sender as TAction).Checked;
end;

procedure TCEMainForm.ViewStatusBarExecute(Sender: TObject);
begin
  StatusBar.Visible := (Sender as TAction).Checked;
end;

procedure TCEMainForm.ActionFileNewExecute(Sender: TObject);
begin
  if CodeEdit.Modified then begin
    case MessageDlg('The current file has not been saved.  Do you want to save it?',
                   mtWarning, [mbYes, mbNo, mbCancel], 0) of
      id_Yes:   begin
                  SaveFileExecute(Self);
                  if CodeEdit.Modified then
                    if MessageDlg('Do you want to discard changes in current file?',
                       mtConfirmation, [mbYes, mbNo], 0) = mrNo then Exit
                  else
                    Exit;
                end;
      id_Cancel: Exit;
    end;
  end;
  CodeEdit.ClearAll;
  CodeEdit.SetSavePoint;
  Caption := 'Code Editor - Untitled';
  CodeEdit.ReadOnly := False;
end;

procedure TCEMainForm.SaveFileExecute(Sender: TObject);
begin
  if (FileName = '') or AnsiContainsText(FileName, 'Untitled') then
    FileSaveAs.Execute
  else begin
    CodeEdit.SaveToFile(FileName);
    CodeEdit.Modified := False;
  end;
end;

procedure TCEMainForm.FileSaveAsAccept(Sender: TObject);
begin
  CodeEdit.SaveToFile(FileSaveAs.Dialog.FileName);
  FileName := FileSaveAs.Dialog.FileName;
  CodeEdit.Modified := False;
  Caption := 'Code Editor -' + FileName;
end;

procedure TCEMainForm.FileOpenAccept(Sender: TObject);
Var
  Ext : string;
begin
  if CodeEdit.Modified then begin
    case MessageDlg('The current file has not been saved.  Do you want to save it?',
                   mtWarning, [mbYes, mbNo, mbCancel], 0) of
      id_Yes:   begin
                  SaveFileExecute(Self);
                  if CodeEdit.Modified then
                    if MessageDlg('Do you want to discard changes in current file?',
                       mtConfirmation, [mbYes, mbNo], 0) = mrNo then Exit
                  else
                    Exit;
                end;
      id_Cancel: Exit;
    end;
  end;
  CodeEdit.LoadFromFile(FileOpen.Dialog.FileName);
  FileName := FileOpen.Dialog.FileName;
  Caption := 'Code Editor - ' + FileName;
  CodeEdit.ReadOnly := False;

  // Set the Syntax;
  Ext := ExtractFileExt(FileName);
  //C and CPP files|*.c;*.cc;*.cpp;*.cxx;*.cs;*.h;*.hh;*.hpp;*.hxx|
  if (AnsiCompareText(Ext, '.c') = 0) or
     (AnsiCompareText(Ext, '.cc') = 0) or
     (AnsiCompareText(Ext, '.cpp') = 0) or
     (AnsiCompareText(Ext, '.cxx') = 0) or
     (AnsiCompareText(Ext, '.cs') = 0) or
     (AnsiCompareText(Ext, '.h') = 0) or
     (AnsiCompareText(Ext, '.hh') = 0) or
     (AnsiCompareText(Ext, '.hpp') = 0) or
     (AnsiCompareText(Ext, '.hxx') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_CPP;
  //Java and Jscript|*.java;*.js|
  end else if (AnsiCompareText(Ext, '.js') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_CPP;
    CodeEdit.Highlighter.SetKeywords(JavaScriptKeywords);
  end else if (AnsiCompareText(Ext, '.java') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_CPP;
    CodeEdit.Highlighter.SetKeywords(JavaKeywords);
  //Pascal files (*.pas)|*.PAS;*.p|
  end else if (AnsiCompareText(Ext, '.pas') = 0) or
     (AnsiCompareText(Ext, '.p') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_PASCAL;
  //VB and VBScript files (*.vbs)|*.VBS;*.vbs;*.bas|
  end else if (AnsiCompareText(Ext, '.vbs') = 0) or
     (AnsiCompareText(Ext, '.bas') = 0) or
     (AnsiCompareText(Ext, '.vb') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_VB;
  //Python files (*.py)|*.py;*.pyw|
  end else if (AnsiCompareText(Ext, '.py') = 0) or
     (AnsiCompareText(Ext, '.pyw') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_Python;
  //Ruby files (*.rb)|*.rb|
  end else if (AnsiCompareText(Ext, '.rb') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_Ruby;
  //HTML files|*.html;*.htm;*.asp;*.shtml;*.htd|
  end else if (AnsiCompareText(Ext, '.html') = 0) or
     (AnsiCompareText(Ext, '.htm') = 0) or
     (AnsiCompareText(Ext, '.asp') = 0) or
     (AnsiCompareText(Ext, '.shtml') = 0) or
     (AnsiCompareText(Ext, '.htd') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_HTML;
  //XML files|*.xml;*.xsl;*.svg;*.xul;*.xsd;*.dtd;*.xslt;*.axl|
  end else if (AnsiCompareText(Ext, '.xml') = 0) or
     (AnsiCompareText(Ext, '.xsl') = 0) or
     (AnsiCompareText(Ext, '.svg') = 0) or
     (AnsiCompareText(Ext, '.xul') = 0) or
     (AnsiCompareText(Ext, '.xsd') = 0) or
     (AnsiCompareText(Ext, '.dtd') = 0) or
     (AnsiCompareText(Ext, '.xslt') = 0) or
     (AnsiCompareText(Ext, '.axl') = 0) then begin
    CodeEdit.Highlighter.Language := Syn_XML;
  end else
    CodeEdit.Highlighter.Language := Syn_NULL;

  //  Enable Code Folding except for text
  if CodeEdit.Highlighter.Language = Syn_NULL then
    CodeEdit.CodeFolding := False
  else
    CodeEdit.CodeFolding := True;
end;

procedure TCEMainForm.ReDoExecute(Sender: TObject);
begin
  CodeEdit.Redo;
end;

procedure TCEMainForm.ReDoUpdate(Sender: TObject);
begin
  Redo.Enabled := CodeEdit.CanRedo;
end;

procedure TCEMainForm.ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
Var
  Pos : integer;
begin
  //FindNext action
  SearchFindNext.Enabled := SciSearchReplace.SearchText <> '';
  //FindReplace action
  SearchReplace.Enabled := not CodeEdit.ReadOnly;

  //Status bar
  //Panel 0 for Hints and Messages
  //Panel 1 for Position
  with CodeEdit do begin
    Pos := CodeEdit.GetCurrentPos;
    StatusBar.Panels[1].Text := Format('%d:%d', [LineFromPosition(Pos)+1, GetColumn(Pos)+1]);
  end;

  //Paner 2 for Modifed
  if CodeEdit.Modified then
    StatusBar.Panels[2].Text := 'Modified'
  else
    StatusBar.Panels[2].Text := '';

  //Panel 3 for Insert/Overwrite
  if CodeEdit.GetOvertype  then
    StatusBar.Panels[3].Text := 'Overwrite'
  else
    StatusBar.Panels[3].Text := 'Insert';

  //Panel 4 for CAPS
  if Lo(GetKeyState(VK_CAPITAL)) = 1 then
    StatusBar.Panels[4].Text := 'CAPS'
  else
    StatusBar.Panels[4].Text := '';

  Done := True;
end;

procedure TCEMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  DialogValue : integer;
begin
  if CodeEdit.Modified then begin
    DialogValue := MessageDlg('The current file has not been saved.  Do you want to save it?',
                   mtWarning, [mbYes, mbNo, mbCancel], 0);
    case DialogValue of
      id_Yes:   begin
                  SaveFileExecute(Self);
                  if CodeEdit.Modified then CanClose := False
                  else CanClose := True;
                end;
      id_No: CanClose := True;
      id_Cancel: CanClose := False;
    end;
  end else
    CanClose := True;           
end;

procedure TCEMainForm.ZoomOutExecute(Sender: TObject);
begin
  CodeEdit.ZoomOut;
end;

procedure TCEMainForm.ZoomInExecute(Sender: TObject);
begin
  CodeEdit.ZoomIn;
end;

procedure TCEMainForm.CodeFoldAllExecute(Sender: TObject);
begin
  CodeEdit.FoldAll;
end;

procedure TCEMainForm.ToggleBookMarkExecute(Sender: TObject);
Var
  LineNo : integer;
  State : integer;
begin
  LineNo := CodeEdit.LineFromPosition(CodeEdit.GetCurrentPos);
  State := CodeEdit.MarkerGet(LineNo);
  if (State and (1 shl MarkerBookmark)) > 0 then
    CodeEdit.MarkerDelete(LineNo, MarkerBookmark)
  else
    CodeEdit.MarkerAdd(LineNo, MarkerBookmark);
end;

procedure TCEMainForm.NextBookmarkExecute(Sender: TObject);
Var
  LineNo : integer;
  NextLine : integer;
begin
  LineNo := CodeEdit.LineFromPosition(CodeEdit.GetCurrentPos);
  NextLine := CodeEdit.MarkerNext(LineNo+1, 1 shl MarkerBookMark);
  if NextLine < 0 then
    NextLine := CodeEdit.MarkerNext(0, 1 shl MarkerBookMark);
  if (NextLine < 0) or (NextLine = LineNo) then begin
    StatusBar.Panels[0].Text := 'No other bookmark';
    Beep;
  end else begin
    CodeEdit.EnsureVisibleEnforcePolicy(NextLine);
    CodeEdit.GotoLine(NextLine);
  end;
end;

procedure TCEMainForm.PreviousBookMarkExecute(Sender: TObject);
Var
  LineNo : integer;
  LineCount : integer;
  PrevLine : integer;
begin
  LineNo := CodeEdit.LineFromPosition(CodeEdit.GetCurrentPos);
  LineCount := CodeEdit.GetLineCount;
  PrevLine := CodeEdit.MarkerPrevious(LineNo-1, 1 shl MarkerBookMark);
  if PrevLine < 0 then
    PrevLine := CodeEdit.MarkerPrevious(LineCount, 1 shl MarkerBookMark);
  if (PrevLine < 0) or (PrevLine = LineNo) then begin
    StatusBar.Panels[0].Text := 'No other bookmark';
    Beep;
  end else begin
    CodeEdit.EnsureVisibleEnforcePolicy(PrevLine);
    CodeEdit.GotoLine(PrevLine);
  end;
end;

procedure TCEMainForm.SyntaxExecute(Sender: TObject);
begin
  with Sender as TAction do begin
    CodeEdit.Highlighter.Language := TSCLEX(Tag);
    if Tag = 1 then
      CodeEdit.CodeFolding := False
    else
      CodeEdit.CodeFolding := True;
    Checked := True;
  end;
end;

procedure TCEMainForm.SyntaxUpdate(Sender: TObject);
begin
  with Sender as TAction do
    Checked := CodeEdit.Highlighter.Language = TSCLEX(Tag);
end;

procedure TCEMainForm.SearchFindExecute(Sender: TObject);
begin
  Statusbar.Panels[0].Text := '';
  SciSearchReplace.ShowSearchReplaceDialog(False);
end;

procedure TCEMainForm.SearchFindNextExecute(Sender: TObject);
begin
  Statusbar.Panels[0].Text := '';
  SciSearchReplace.DoSearchReplaceText(False, SciSearchReplace.SearchBackwards);
end;

procedure TCEMainForm.SearchReplaceExecute(Sender: TObject);
begin
  Statusbar.Panels[0].Text := '';
  SciSearchReplace.ShowSearchReplaceDialog(True);
end;

procedure TCEMainForm.SearchBracesExecute(Sender: TObject);
begin
  CodeEdit.GoMatchingBrace(False, True);
end;

procedure TCEMainForm.FileExportToHTMLExecute(Sender: TObject);
Var
  FileStream : TFileStream;
begin
  if HTMLSave.Execute then begin
    FileStream := TFileStream.Create(HTMLSave.FileName, fmCreate);
    try
      CodeEdit.ExportToHTML(FileStream, FileName);
    finally
      FileStream.Free
    end;
  end;
end;

procedure TCEMainForm.ContentsToWebBrowser;
Var
  MS : TMemoryStream;
begin
  MS := TMemoryStream.Create;
  try
    CodeEdit.ExportToHTML(MS, FileName);
    MS.Position := 0;
    WebBrowser.Navigate('about:blank');
    (WebBrowser.Document as IPersistStreamInit).Load(TStreamAdapter.Create(MS));
  finally
    MS.Free;
  end;
end;

procedure TCEMainForm.FilePrintExecute(Sender: TObject);
begin
  ContentsToWebBrowser;
  while WebBrowser.QueryStatusWB(OLECMDID_PRINT) <> OLECMDF_SUPPORTED + OLECMDF_ENABLED do
    Forms.Application.ProcessMessages;
  WebBrowser.ExecWB(OLECMDID_PRINT, OLECMDEXECOPT_PROMPTUSER);
end;

procedure TCEMainForm.FilePrintPreviewExecute(Sender: TObject);
begin
  ContentsToWebBrowser;
  while WebBrowser.QueryStatusWB(OLECMDID_PRINT) <> OLECMDF_SUPPORTED + OLECMDF_ENABLED do
    Forms.Application.ProcessMessages;
  WebBrowser.ExecWB(OLECMDID_PRINTPREVIEW, OLECMDEXECOPT_DODEFAULT);
end;

procedure TCEMainForm.PageSetupExecute(Sender: TObject);
begin
  ContentsToWebBrowser;
  while WebBrowser.QueryStatusWB(OLECMDID_PRINT) <> OLECMDF_SUPPORTED + OLECMDF_ENABLED do
    Forms.Application.ProcessMessages;
  WebBrowser.ExecWB(OLECMDID_PAGESETUP, OLECMDEXECOPT_DODEFAULT);
end;

procedure TCEMainForm.EditorOptionsExecute(Sender: TObject);
begin
  OptionsDialog.Execute;
end;

procedure TCEMainForm.SciSearchReplaceTextNotFound(Sender: TObject);
begin
  MessageBeep(MB_ICONASTERISK);
  Statusbar.Panels[0].Text := 'Text not found';
end;

end.
