unit EdOptionsWin;
////////////////////////////////////////////////////////////////////////////////
// Unit    : EdOptionsWin
// Purpose : Options Dialog for Scintilla editors
// Created : 20/03/2003
// Author  : Kiriakos Vlahos (kvlahos@london.edu)
////////////////////////////////////////////////////////////////////////////////
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, JvEdit, SciLexer, SciKeyBindings,
  ComCtrls, JvSpin, JvCombobox, JvColorCombo, Mask, JvMaskEdit,
  ScintillaSynLexers, JvExStdCtrls, JvExMask, Menus;

type

  TEditorOptions = class(TPersistent)
  private
    FReadOnly : boolean;
    FBraceHilite : boolean;
    FGutter: boolean;
    FWordWrap: boolean;
    FLineNumbers: boolean;
    FIndentation : TIndentationOptions;
    FCodeFolding: boolean;
    FUseUnicode: boolean;
    FIndentWidth: integer;
    FTabWidth: integer;
    FEOLStyle: TEOLStyle;
    FFont: TFont;
    FSynLexer: TSciSynLexer;
    fKeyCommands : TSciKeyCommandCollection;
    procedure SetFont(const Value: TFont);
    procedure SetSynLexer(const Value: TSciSynLexer);
    procedure SetKeyCommands(const Value: TSciKeyCommandCollection);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent);override;
    procedure GetOptions(Scintilla : TScintilla);
    procedure SetOptions(Scintilla : TScintilla);
  published
    property SynLexer : TSciSynLexer read FSynLexer write SetSynLexer;
    property ReadOnly : boolean read fReadOnly write fReadOnly default False;
    property BraceHilite : boolean read FBraceHilite write FBraceHilite default True;
    property Gutter : boolean read FGutter write FGutter default True;
    property Indentation : TIndentationOptions read FIndentation write fIndentation;
    property LineNumbers : boolean read FLineNumbers write FLineNumbers default False;
    property UseUnicode : boolean read FUseUnicode write FUseUnicode default True;
    property WordWrap : boolean read FWordWrap write FWordWrap default False;
    property CodeFolding : boolean read FCodeFolding write FCodeFolding default True;
    property EOLStyle : TEOLStyle read FEOLStyle write FEOLStyle default EOL_CRLF;
    property TabWidth : integer read FTabWidth write FTabWidth  default 8;
    property IndentWidth : integer read FIndentWidth write FIndentWidth default 0;
    property KeyCommands : TSciKeyCommandCollection read FKeyCommands write SetKeyCommands;
    property Font : TFont read FFont write SetFont;
  end;


  TEdOptionsWindow = class(TForm)
    Panel1: TPanel;
    OptionPages: TPageControl;
    OptionsPage: TTabSheet;
    Panel2: TPanel;
    GroupBox1: TGroupBox;
    CodeFoldingCB: TCheckBox;
    GutterCB: TCheckBox;
    LineNumbersCB: TCheckBox;
    IndentationGuidesCB: TCheckBox;
    FontButton: TBitBtn;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    KeepIndentCB: TCheckBox;
    TabIndentsCB: TCheckBox;
    BackSpaceUnIndentsCB: TCheckBox;
    GroupBox3: TGroupBox;
    Label3: TLabel;
    WordWrapCB: TCheckBox;
    UseUnicodeCB: TCheckBox;
    EOLStyleCB: TComboBox;
    Panel3: TPanel;
    CancelBtn: TBitBtn;
    ApplyBtn: TBitBtn;
    OKBtn: TBitBtn;
    HighlighterPage: TTabSheet;
    Panel4: TPanel;
    FontDialog: TFontDialog;
    ReadOnlyCB: TCheckBox;
    GroupBox5: TGroupBox;
    StylesLB: TListBox;
    GroupBox6: TGroupBox;
    LanguageCB: TComboBox;
    GroupBox7: TGroupBox;
    Label4: TLabel;
    StyleNumberSE: TJvSpinEdit;
    DescriptionEB: TEdit;
    Label9: TLabel;
    AddStyleB: TSpeedButton;
    DeleteStyleB: TSpeedButton;
    KeywordsPage: TTabSheet;
    Panel5: TPanel;
    BraceHiliteCB: TCheckBox;
    StyleDefGB: TGroupBox;
    Label5: TLabel;
    FontCB: TJvFontComboBox;
    DefaultFontCB: TCheckBox;
    SizeSE: TJvSpinEdit;
    Label6: TLabel;
    Label7: TLabel;
    ForeCB: TColorBox;
    Label8: TLabel;
    BackCB: TColorBox;
    ItalicCB: TCheckBox;
    BoldCB: TCheckBox;
    UnderlineCB: TCheckBox;
    VisibleCB: TCheckBox;
    ChangeableCB: TCheckBox;
    EOLFilledCB: TCheckBox;
    Label10: TLabel;
    CaseCB: TComboBox;
    GroupBox4: TGroupBox;
    KeyListsLB: TListBox;
    KeywordListGB: TGroupBox;
    Label11: TLabel;
    Label12: TLabel;
    KeyListAdd: TSpeedButton;
    KeyListDelete: TSpeedButton;
    KeyListNumberSE: TJvSpinEdit;
    KeyListDescriptionEB: TEdit;
    GroupBox9: TGroupBox;
    KeywordsM: TMemo;
    KeyCommandsPage: TTabSheet;
    Panel6: TPanel;
    pnlCommands: TPanel;
    KeyCmdList: TListView;
    btnAdd: TButton;
    btnEdit: TButton;
    btnDelete: TButton;
    btnReset: TButton;
    TabWidthSE: TJvSpinEdit;
    IndentWidthSE: TJvSpinEdit;
    HotspotCB: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ApplyBtnClick(Sender: TObject);
    procedure FontButtonClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure StylesLBClick(Sender: TObject);
    procedure LanguageCBChange(Sender: TObject);
    procedure StyleElementChanged(Sender: TObject);
    procedure DeleteStyleBClick(Sender: TObject);
    procedure AddStyleBClick(Sender: TObject);
    procedure KeyListsLBClick(Sender: TObject);
    procedure KeyListElementsChange(Sender: TObject);
    procedure KeyListDeleteClick(Sender: TObject);
    procedure KeyListAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
  private
    { Private declarations }
    FScintilla : TScintilla;
    fEditorOptions : TEditorOptions;
    fOldEditorOptions : TEditorOptions;
    UpdatingStyle : Boolean;
    UpdatingKeyList : Boolean;
    procedure SetScintilla(const Value: TScintilla);
    procedure UpdateStyles;
    procedure UpdateKeywords;
    procedure UpdateKeyCommands;
  public
    { Public declarations }
    property Scintilla : TScintilla read fScintilla write SetScintilla;
  end;

var
  EdOptionsWindow: TEdOptionsWindow;

implementation

{$R *.dfm}

Uses
  SciKeyEditForm;

{ TEditorOptions }

procedure TEditorOptions.Assign(Source: TPersistent);
begin
  if Source is TEditorOptions then with TEditorOptions(Source) do begin
    Self.FReadOnly := ReadOnly;
    Self.FBraceHilite := BraceHilite;
    Self.FGutter := Gutter;
    Self.FWordWrap := WordWrap;
    Self.FLineNumbers := LineNumbers;
    Self.FIndentation := Indentation;
    Self.FCodeFolding := CodeFolding;
    Self.FUseUnicode := UseUnicode;
    Self.FTabWidth := TabWidth ;
    Self.FIndentWidth := IndentWidth;
    Self.FEOLStyle := EOLStyle;
    Self.SetFont(Font);
    Self.SynLexer.Assign(SynLexer);
    Self.KeyCommands.Assign(KeyCommands);
  end else if Source is TScintilla then begin
    GetOptions(Source as TScintilla);
  end else
    inherited;
end;

constructor TEditorOptions.Create;
begin
  inherited;
  FFont := TFont.Create;
  FSynLexer := TSciSynLexer.Create(nil);
  fKeyCommands := TSciKeyCommandCollection.Create(nil);
end;

destructor TEditorOptions.Destroy;
begin
  FFont.Free;
  FSynLexer.Free;
  inherited;
end;

procedure TEditorOptions.GetOptions(Scintilla: TScintilla);
begin
  with Scintilla do begin
    FReadOnly := ReadOnly;
    FBraceHilite := BraceHilite;
    FGutter := Gutter1.Width <> 0;
    FWordWrap := WordWrap;
    FLineNumbers := Gutter0.Width <> 0;
    FIndentation := Indentation;
    FCodeFolding := CodeFolding;
    FUseUnicode := UseUnicode;
    FTabWidth := TabWidth ;
    FIndentWidth := IndentWidth;
    FEOLStyle := EOLStyle;
    Self.SetFont(Font);
    SynLexer.Assign(Highlighter);
    fKeyCommands.Assign(Scintilla.KeyCommands);
  end;
end;

procedure TEditorOptions.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TEditorOptions.SetKeyCommands(
  const Value: TSciKeyCommandCollection);
begin
  FKeyCommands.Assign(Value);
end;

procedure TEditorOptions.SetOptions(Scintilla: TScintilla);
begin
  with Scintilla do begin
    if ReadOnly <> FReadOnly then ReadOnly := FReadOnly;
    BraceHilite := FBraceHilite;
    if FGutter <> (Gutter1.Width <> 0) then begin
      if FGutter then
        Gutter1.Width := 16
      else
        Gutter1.Width := 0;
    end;
    if WordWrap <> FWordWrap then WordWrap := FWordWrap;
    if FLineNumbers <> (Gutter0.Width <> 0) then  begin
      if FLineNumbers then begin
        Gutter0.Width := 32;
        Gutter0.MarginType := GUT_LINENUMBER;
      end else
        Gutter0.Width := 0;
    end;
    if Indentation <> FIndentation then Indentation := FIndentation;
    if CodeFolding <> FCodeFolding then CodeFolding := FCodeFolding;
    if UseUnicode <> FUseUnicode then UseUnicode := FUseUnicode;
    if IndentWidth <> FIndentWidth then IndentWidth := FIndentWidth;
    if TabWidth <> FTabWidth then TabWidth := FTabWidth;
    if EOLStyle <> FEOLStyle then EOLStyle := FEOLStyle;
    if (Font.Name <> FFont.Name) or (Font.Size <> FFont.Size) or
       (Font.Charset <> FFont.Charset) or (Font.Style <> FFont.Style) then
       Font := FFont;
    Highlighter.Assign(Self.SynLexer);
    KeyCommands.Assign(FKeyCommands);
  end;
end;


procedure TEditorOptions.SetSynLexer(const Value: TSciSynLexer);
begin
  FSynLexer.Assign(Value);
end;

{ TEdOptionsWindow }

procedure TEdOptionsWindow.SetScintilla(const Value: TScintilla);
begin
  fScintilla := Value;
  fEditorOptions.GetOptions(Scintilla);
  with fEditorOptions do begin
    ReadOnlyCB.Checked := ReadOnly;
    BraceHiliteCB.Checked := BraceHilite;
    GutterCB.Checked := Gutter;
    WordWrapCB.Checked := WordWrap;
    LineNumbersCB.Checked := LineNumbers;
    KeepIndentCB.Checked := SciLexer.KeepIndent in Indentation;
    TabIndentsCB.Checked := SciLexer.TabIndents in Indentation;;
    BackSpaceUnindentsCB.Checked := SciLexer.BackSpaceUnindents in Indentation;
    IndentationGuidesCB.Checked := SciLexer.IndentationGuides in Indentation;
    CodeFoldingCB.Checked := CodeFolding;
    UseUnicodeCB.Checked := UseUnicode;
    TabWidthSE.Value := TabWidth;
    IndentWidthSE.Value := IndentWidth;
    EOLStyleCB.ItemIndex := Integer(EOLStyle);
    //  Synax Highlighter
    LanguageCB.ItemIndex := Integer(SynLexer.Language) - 1;
    UpdateStyles;
    UpdateKeywords;
    UpdateKeyCommands;
  end;
  fOldEditorOptions.Assign(FEditorOptions);
end;

procedure TEdOptionsWindow.FormCreate(Sender: TObject);
begin
  fEditorOptions := TEditorOptions.Create;
  fOldEditorOptions := TEditorOptions.Create;
end;

procedure TEdOptionsWindow.FormDestroy(Sender: TObject);
begin
  fEditorOptions.Free;
  fOldEditorOptions.Free;
end;

procedure TEdOptionsWindow.ApplyBtnClick(Sender: TObject);
begin
  with fEditorOptions do begin
    ReadOnly := ReadOnlyCB.Checked;
    BraceHilite := BraceHiliteCB.Checked;
    Gutter := GutterCB.Checked;
    WordWrap := WordWrapCB.Checked;
    LineNumbers := LineNumbersCB.Checked;
    Indentation := [];
    if KeepIndentCB.Checked then Indentation := Indentation + [SciLexer.KeepIndent];
    if TabIndentsCB.Checked then Indentation := Indentation + [SciLexer.TabIndents];
    if BackSpaceUnindentsCB.Checked then Indentation := Indentation + [SciLexer.BackSpaceUnindents];
    if IndentationGuidesCB.Checked then Indentation := Indentation + [SciLexer.IndentationGuides];
    CodeFolding := CodeFoldingCB.Checked;
    UseUnicode := UseUnicodeCB.Checked;
    TabWidth := Round(TabWidthSE.Value);
    IndentWidth := Round(IndentWidthSE.Value);
    EOLStyle := TEOLStyle(EOLStyleCB.ItemIndex);

    if Assigned(Scintilla) then SetOptions(FScintilla);
  end;
end;

procedure TEdOptionsWindow.FontButtonClick(Sender: TObject);
begin
  FontDialog.Font := fEditorOptions.Font;
  if FontDialog.Execute then
    fEditorOptions.Font := FontDialog.Font;
end;

procedure TEdOptionsWindow.OKBtnClick(Sender: TObject);
begin
  ApplyBtnClick(Self);
end;

procedure TEdOptionsWindow.CancelBtnClick(Sender: TObject);
begin
  if not Assigned(Scintilla) then Exit;
  fOldEditorOptions.SetOptions(FScintilla);
end;

procedure TEdOptionsWindow.StylesLBClick(Sender: TObject);
Var
  SciStyle : TSciStyle;
begin
  if StylesLB.ItemIndex >= 0 then begin
    // To avoid getting into StyleElementChanged
    UpdatingStyle := True;

    SciStyle := FEditorOptions.SynLexer.StyleList.Items[StylesLB.ItemIndex] as TSciStyle;
    StyleNumberSE.Value := SciStyle.StyleNumber;
    DescriptionEB.Text := SciStyle.Name;
    if SciStyle.FontName = '' then begin
      FontCB.FontName := '';
      DefaultFontCB.Checked := True;
    end else begin
      FontCB.FontName := SciStyle.FontName;
      DefaultFontCB.Checked := False;
    end;
    SizeSE.Value := SciStyle.FontSize;
    ForeCB.Selected := SciStyle.ForeColor;
    ForeCB.Repaint;
    BackCB.Selected := SciStyle.BackColor;
    BackCB.Repaint;
    CaseCB.ItemIndex := Integer(SciStyle.CharCase);
    ItalicCB.Checked := fsItalic in SciStyle.FontStyles;
    BoldCB.Checked := fsBold in SciStyle.FontStyles;
    UnderlineCB.Checked := fsUnderline in SciStyle.FontStyles;
    VisibleCB.Checked := SciStyle.Visible;
    ChangeableCB.Checked := SciStyle.Changeable;
    EOLFilledCB.Checked := SciStyle.EOLFilled;
    HotspotCB.Checked := SciStyle.Hotspot;
    // Now set it to false
    UpdatingStyle := False;
  end;
end;

procedure TEdOptionsWindow.LanguageCBChange(Sender: TObject);
begin
  fEditorOptions.SynLexer.Language := TSCLEX(LanguageCB.ItemIndex+ 1);
  UpdateStyles;
  UpdateKeywords;
end;

procedure TEdOptionsWindow.UpdateStyles;
Var
  i : integer;
begin
  with FEditorOptions do begin
    StylesLB.Items.BeginUpdate;
    try
      StylesLB.Items.Clear;
      for i := 0 to FSynLexer.StyleList.Count - 1 do
        with FSynLexer.StyleList.Items[i] as TSciStyle do
          StylesLB.Items.Add(Name);
    finally
      StylesLB.Items.EndUpdate;
    end;
    if StylesLB.Count > 0 then begin
      StyleDefGB.Enabled := True;
      StylesLB.ItemIndex := 0;
      StylesLBClick(Self);
    end else
      StyleDefGB.Enabled := False;
  end;
end;

procedure TEdOptionsWindow.StyleElementChanged(Sender: TObject);
Var
  SciStyle : TSciStyle;
begin
  if UpdatingStyle then Exit;
  if StylesLB.ItemIndex >= 0 then begin
    SciStyle := FEditorOptions.SynLexer.StyleList.Items[StylesLB.ItemIndex] as TSciStyle;
    SciStyle.StyleNumber := Trunc(StyleNumberSE.Value);
    SciStyle.Name := DescriptionEB.Text;
    StylesLB.Items[StylesLB.ItemIndex] := DescriptionEB.Text;
    if DefaultFontCB.Checked then
      SciStyle.FontName := ''
    else
      SciStyle.FontName := FontCB.FontName;
    SciStyle.FontSize := Trunc(SizeSE.Value);
    SciStyle.ForeColor := ForeCB.Selected;
    SciStyle.BackColor := BackCB.Selected;
    SciStyle.CharCase := TSciCase(CaseCB.ItemIndex);
    SciStyle.FontStyles := [];
    if ItalicCB.Checked then SciStyle.FontStyles := SciStyle.FontStyles + [fsItalic];
    if BoldCB.Checked then SciStyle.FontStyles := SciStyle.FontStyles + [fsBold];
    if UnderlineCB.Checked then SciStyle.FontStyles := SciStyle.FontStyles + [fsUnderline];
    SciStyle.Visible := VisibleCB.Checked;
    SciStyle.Changeable := ChangeableCB.Checked;
    SciStyle.EOLFilled := EOLFilledCB.Checked;
    SciStyle.Hotspot := HotspotCB.Checked;
  end;
end;

procedure TEdOptionsWindow.DeleteStyleBClick(Sender: TObject);
begin
  if StylesLB.ItemIndex >= 0 then begin
    (FEditorOptions.SynLexer.StyleList.Items[StylesLB.ItemIndex] as TSciStyle).Free;
    StylesLB.Items.Delete(StylesLB.ItemIndex);
    UpdateStyles;
  end;
end;

procedure TEdOptionsWindow.AddStyleBClick(Sender: TObject);
begin
  with FEditorOptions.SynLexer.StyleList.Add as TSciStyle do begin
    Name := 'New Style';
    StyleNumber :=  FEditorOptions.SynLexer.StyleList.Count - 1;
  end;
  UpdateStyles;
  StylesLB.Selected[StylesLB.Items.Count-1] := True;
  StylesLBClick(Self);
end;

procedure TEdOptionsWindow.UpdateKeywords;
Var
  i : integer;
begin
  with FEditorOptions do begin
    KeyListsLB.Items.BeginUpdate;
    try
      KeyListsLB.Items.Clear;
      for i := 0 to FSynLexer.KeywordsList.Count - 1 do
        with FSynLexer.KeywordsList.Items[i] as TSciKeywords do
          KeyListsLB.Items.Add(Name);
    finally
      KeyListsLB.Items.EndUpdate;
    end;
    if KeyListsLB.Count > 0 then begin
      KeywordsM.Enabled := True;
      KeyListsLB.ItemIndex := 0;
      KeyListsLBClick(Self);
    end else begin
      KeywordsM.Text := '';
      KeywordsM.Enabled := False;
    end;
  end;
end;

procedure TEdOptionsWindow.KeyListsLBClick(Sender: TObject);
var
  SciKeywords : TSciKeywords;
begin
  if KeyListsLB.ItemIndex >= 0 then begin
    // To avoid getting into KeyListElementChanged
    UpdatingKeyList := True;

    SciKeywords := FEditorOptions.SynLexer.KeywordsList.Items[KeyListsLB.ItemIndex] as TSciKeywords;
    KeyListNumberSE.Value := SciKeywords.KeywordListNumber;
    KeyListDescriptionEB.Text := SciKeywords.Name;
    SciKeywords.Keywords.Delimiter := ' ';
    KeywordsM.Text := SciKeywords.Keywords.DelimitedText;

    UpdatingKeyList := False;
  end;
end;

procedure TEdOptionsWindow.KeyListElementsChange(Sender: TObject);
Var
  SciKeywords : TSciKeywords;
begin
  if UpdatingKeyList then Exit;
  if KeyListsLB.ItemIndex >= 0 then begin
    SciKeywords := FEditorOptions.SynLexer.KeywordsList.Items[KeyListsLB.ItemIndex] as TSciKeywords;
    SciKeywords.Name := KeyListDescriptionEB.Text;
    SciKeywords.KeywordListNumber := Trunc(KeyListNumberSE.Value);
    SciKeywords.Keywords.Delimiter := ' ';
    SciKeywords.Keywords.DelimitedText := KeywordsM.Text;
  end;
end;

procedure TEdOptionsWindow.KeyListDeleteClick(Sender: TObject);
begin
  if KeyListsLB.ItemIndex >= 0 then begin
    (FEditorOptions.SynLexer.KeywordsList.Items[KeyListsLB.ItemIndex] as TSciKeywords).Free;
    KeyListsLB.Items.Delete(KeyListsLB.ItemIndex);
    UpdateKeywords;
  end;
end;

procedure TEdOptionsWindow.KeyListAddClick(Sender: TObject);
begin
  with FEditorOptions.SynLexer.KeywordsList.Add as TSciKeywords do begin
    Name := 'New Key List';
    KeywordListNumber :=  FEditorOptions.SynLexer.KeywordsList.Count-1;
  end;
  UpdateKeywords;
  KeyListsLB.Selected[KeyListsLB.Items.Count-1] := True;
  KeyListsLBClick(Self);
end;

procedure TEdOptionsWindow.UpdateKeyCommands;
Var
  i : integer;
  KeyCommand : TSciKeyCommand;
  Item : TListItem;
  Ident : string;
begin
  KeyCmdList.Items.BeginUpdate;
  try
    KeyCmdList.Clear;
    for i := 0 to fEditorOptions.fKeyCommands.Count - 1 do begin
      KeyCommand := fEditorOptions.fKeyCommands.Items[i] as TSciKeyCommand;
      Item := KeyCmdList.Items.Add;
      Item.Caption :=ShortCutToText(KeyCommand.ShortCut);
      if IntToIdent(KeyCommand.Command, Ident, Sci_KeyboardCommandMap) then
        Item.SubItems.Add(Ident);
      Item.Data := KeyCommand;
    end;
  finally
    KeyCmdList.Items.EndUpdate;
  end;
end;

procedure TEdOptionsWindow.btnDeleteClick(Sender: TObject);
begin
  if Assigned(KeyCmdList.Selected) then begin
    TSciKeyCommand(KeyCmdList.Selected.Data).Free;
    UpdateKeyCommands;
  end;
end;

procedure TEdOptionsWindow.btnResetClick(Sender: TObject);
begin
  fEditorOptions.fKeyCommands.ResetDefaultCommands;
  UpdateKeyCommands;
end;

procedure TEdOptionsWindow.btnEditClick(Sender: TObject);
Var
  KeyCommand : TSciKeyCommand;
begin
  if Assigned(KeyCmdList.Selected) then begin
    KeyCommand := TSciKeyCommand(KeyCmdList.Selected.Data);
    with TKeyEditForm.Create(Self) do begin
      cmbCommand.ItemIndex :=  cmbCommand.Items.IndexOf(KeyCmdList.Selected.SubItems[0]);
      HotKey.HotKey := KeyCommand.ShortCut;
      HotKey.Modifiers := HotKey.Modifiers + [hkExt];
      if (ShowModal = mrOK) and (cmbCommand.ItemIndex >= 0) and
         (HotKey.HotKey <> 0) then
      begin
        KeyCommand.ShortCut := HotKey.HotKey;
        KeyCommand.Command :=  Integer(cmbCommand.Items.Objects[cmbCommand.ItemIndex]);
        UpdateKeyCommands;
      end;
      Release;
    end;
  end;
end;


procedure TEdOptionsWindow.btnAddClick(Sender: TObject);
Var
  KeyCommand : TSciKeyCommand;
begin
    with TKeyEditForm.Create(Self) do begin
      if (ShowModal = mrOK) and (cmbCommand.ItemIndex >= 0) and
         (HotKey.HotKey <> 0) then
      begin
        KeyCommand := fEditorOptions.fKeyCommands.FindShortCut(HotKey.HotKey);
        if not Assigned(KeyCommand) then
          KeyCommand := fEditorOptions.fKeyCommands.Add as TSciKeyCommand;
        KeyCommand.ShortCut := HotKey.HotKey;
        KeyCommand.Command :=  Integer(cmbCommand.Items.Objects[cmbCommand.ItemIndex]);
        UpdateKeyCommands;
      end;
      Release;
    end;
end;

end.
