unit SciLexerOptionsDlg;
////////////////////////////////////////////////////////////////////////////////
// Unit    : SciLexerOptionsDlg
// Purpose : Component encapsulating the Options Dialog for Scintilla editors
// Created : 20/03/2003
// Author  : Kiriakos Vlahos (kvlahos@london.edu)
////////////////////////////////////////////////////////////////////////////////

interface

Uses
  SysUtils, Classes, Controls, Forms, SciLexer, EdOptionsWin;

Type
  TSciOptionPage = (opOptions, opHighlighter, opKeywords, opKeyboard);
  TSciOptionPages = set of TSciOptionPage;

  TScintillaOptionsDlg = class(TComponent)
  private
    FScintilla: TScintilla;
    FActivePage: TSciOptionPage;
    FPages: TSciOptionPages;
  protected
    procedure Notification(AComponent: TComponent;
              Operation: TOperation);  override;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute: Boolean;
  published
    property Scintilla: TScintilla read FScintilla write FScintilla;
    property ActivePage: TSciOptionPage read FActivePage write FActivePage default opOptions;
    property Pages: TSciOptionPages read FPages write FPages default [opOptions, opHighlighter, opKeywords, opKeyboard];
  end;


implementation

{ TScintillaOptionsDlg }

constructor TScintillaOptionsDlg.Create(AOwner: TComponent);
begin
  inherited;
  FPages := [opOptions, opHighlighter, opKeywords, opKeyboard];
  FActivePage := opOptions;
end;

function TScintillaOptionsDlg.Execute: Boolean;
begin
  if FScintilla = nil then
    raise Exception.Create('Scintilla property is not assigned');
  with TEdOptionsWindow.Create(Self) do begin
    if not (FActivePage in FPages) then Include(FPages, FActivePage);
    KeywordsPage.TabVisible := opKeywords in FPages;
    HighlighterPage.TabVisible := opHighlighter in FPages;
    OptionsPage.TabVisible := opOptions in FPages;
    KeyCommandsPage.TabVisible := opKeyboard in FPages;
    case FActivePage of
      opOptions : OptionPages.ActivePage := OptionsPage;
      opHighlighter : OptionPages.ActivePage := HighlighterPage;
      opKeywords : OptionPages.ActivePage := KeywordsPage;
      opKeyboard : OptionPages.ActivePage := KeyCommandsPage;
    end;
    Scintilla := FScintilla;
    Result := ShowModal = mrOK;
    Free;
  end;
end;

procedure TScintillaOptionsDlg.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (AComponent = FScintilla) and (Operation = opRemove) then FScintilla := nil;
end;

end.
