unit ScintillaSynLexers;
////////////////////////////////////////////////////////////////////////////////
// Unit    : ScintillaSynLexers
// Purpose : Interface for Scintilla Lexer DLL
// Created : 05/03/2001
// Author  : Kiriakos Vlahos (kvlahos@london.edu)
// History : 15/03/2001 Lexer sturtures and encapsulation of some common
//           Lexers
////////////////////////////////////////////////////////////////////////////////

interface

uses SysUtils, Classes, Graphics, sciSupport;

Type

  TSCLEX = (
      Syn_CONTAINER = 0,
      Syn_NULL = 1,
      Syn_PYTHON = 2,
      Syn_CPP = 3,
      Syn_HTML = 4,
      Syn_XML = 5,
      Syn_PERL = 6,
      Syn_SQL = 7,
      Syn_VB = 8,
      Syn_PROPERTIES = 9,
      Syn_ERRORLIST = 10,
      Syn_MAKEFILE = 11,
      Syn_BATCH = 12,
      Syn_XCODE = 13,
      Syn_LATEX = 14,
      Syn_LUA = 15,
      Syn_DIFF = 16,
      Syn_CONF = 17,
      Syn_PASCAL = 18,
      Syn_AVE = 19,
      Syn_ADA = 20,
      Syn_LISP = 21,
      Syn_RUBY = 22,
      Syn_EIFFEL = 23,
      Syn_EIFFELKW = 24,
      Syn_TCL = 25,
      Syn_NNCRONTAB = 26,
      Syn_BULLANT = 27,
      Syn_VBSCRIPT = 28,
      Syn_ASP = 29,
      Syn_PHP = 30,
      Syn_BAAN = 31,
      Syn_MATLAB = 32,
      Syn_SCRIPTOL = 33,
      Syn_ASM = 34,
      Syn_CPPNOCASE = 35,
      Syn_FORTRAN = 36,
      Syn_F77 = 37,
      Syn_CSS = 38
    );

    TSciCase = (CASE_MIXED = 0,
                CASE_UPPER = 1,
                CASE_LOWER = 2);

  TSciLangKeywords = Record
    Name : string;
    KeywordListNumber : integer;
    KeyWords : string;
  end;

  TSciLangStyle = Record
    StyleNumber : integer;
    Name : string;
    FontName : string;
    FontSize : integer;
    FontStyles : TFontStyles;
    ForeColor : TColor;
    BackColor : TColor;
    CharCase : TSciCase;
    Visible: Boolean;
    Changeable : Boolean;
    Hotspot : Boolean;
    EOLFilled : Boolean;
  end;

  TSciKeywords = class(TCollectionItem)
  private
    fKeyWords : TStrings;
    fName : string;
    fKeywordListNumber : integer;
    procedure SetKeywords(const Value: TStrings);
    procedure SetName(const Value: string);
    procedure SetKeywordListNumber(const Value: integer);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure AssignRec(Rec : TSciLangKeywords);
  published
    property KeywordListNumber : integer read FKeywordListNumber write SetKeywordListNumber;
    property Name : string read fName write SetName;
    property Keywords : TStrings read fKeywords write SetKeywords;
  end;

  TSciStyle = class(TCollectionItem)
  private
    fStyleNumber : integer;
    fName : string;
    fFontName : string;
    fFontSize : integer;
    fFontStyles : TFontStyles;
    fForeColor : TColor;
    fBackColor : TColor;
    fCharCase : TSciCase;
    fVisible: Boolean;
    fChangeable: Boolean;
    fHotspot : boolean;
    fEOLFilled: Boolean;
    procedure SetBackColor(const Value: TColor);
    procedure SetFontName(const Value: string);
    procedure SetFontSize(const Value: integer);
    procedure SetFontStyles(const Value: TFontStyles);
    procedure SetForeColor(const Value: TColor);
    procedure SetName(const Value: string);
    procedure SetStyleNumber(const Value: integer);
    procedure SetCharCase(const Value: TSciCase);
    procedure SetVisible(const Value: Boolean);
    procedure SetChangeable(const Value: Boolean);
    procedure SetEOLFilled(const Value: Boolean);
    procedure SetHotspot(const Value: Boolean);
  public
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
    procedure AssignRec(Rec : TSciLangStyle);
  published
    property FontName : string read fFontName write SetFontName;
    property FontSize : integer read fFontSize write SetFontSize;
    property FontStyles : TFontStyles read FFontStyles write SetFontStyles;
    property ForeColor : TColor read fForeColor write SetForeColor default clDefault;
    property BackColor : TColor read fBackColor write SetBackColor default clDefault;
    property CharCase : TSciCase read fCharCase write SetCharCase;
    property Visible : Boolean read fVisible write SetVisible default true;
    property Changeable : Boolean read fChangeable write SetChangeable default true;
    property EOLFilled : Boolean read fEOLFilled write SetEOLFilled default false;
    property Hotspot : Boolean read fHotspot write SetHotspot default false;
    property Name : string read fName write SetName;
    property StyleNumber : integer read fStyleNumber write SetStyleNumber;
  end;

  TSciKeyWordsList = class(TCollection)
  private
    fEditor : TPersistent; //TScintilla
  public
    procedure SetEditor(Editor : TPersistent);
    procedure Update(Item: TCollectionItem); override;
  end;

  TSciStyleList = class(TCollection)
  private
    fEditor : TPersistent; //TScintilla
  public
    procedure SetEditor(Editor : TPersistent);
    procedure Update(Item: TCollectionItem); override;
  end;

  TSciSynLexer = class (TPersistent)
  private
    fEditor : TPersistent; //TScintilla
    fLanguage : TSCLEX;
    fKeywordsList : TSciKeyWordsList;
    fStyleList : TSciStyleList;
    procedure SetKeywordsList(const Value: TSciKeyWordsList);
    procedure SetLanguage(const Value: TSCLEX);
    procedure SetStyleList(const Value: TSciStyleList);
  protected
    function  GetOwner: TPersistent; override;
  public
    constructor Create(Editor: TPersistent);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Update;
    procedure SetKeywords(Keywords : Array of TSciLangKeywords);
    procedure SetStyles(Styles : Array of TSciLangStyle);
  published
    property Language : TSCLEX read fLanguage write SetLanguage;
    property KeywordsList : TSciKeyWordsList read fKeywordsList write SetKeywordsList;
    property StyleList : TSciStyleList read fStyleList write SetStyleList;
  end;

Var
  PascalKeywords : Array [0..1] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords: 'program const type var begin end array set packed record string if then else ' +
       'while for to downto do with repeat until case of goto exit label procedure ' +
       'function nil file and or not xor div mod unit uses implementation interface ' +
       'external asm inline object constructor destructor virtual far assembler near ' +
       'inherited stdcall cdecl library export exports end. ' +
       'class ansistring raise try except on index name finally ' +
       'resourcestring false true initialization finalization ' +
       'override overload'),
       (Name : 'Class Keywords';
        KeywordListNumber : 1;
        Keywords :  'write read default public protected private property published stored')
     );

  PascalStyles : Array [0..12] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'White space'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Line Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'Doc Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $7F7F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Double quoted string'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Single quoted string'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'Symbols'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $007F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 9; Name : 'Preprocessor'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 10; Name : 'Operators'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 34; Name : 'Brace highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 35; Name : 'Brace Bad highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
     );

  VBKeywords : Array [0..0] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords : 'and begin case call class continue do each else elseif end erase '+
       'error event exit false for function get gosub goto if implement in load loop lset '+
       'me mid new next not nothing on or property raiseevent rem resume return rset '+
       'select set stop sub then to true unload until wend while with withevents '+
       'attribute alias as boolean byref byte byval const compare currency date declare dim double '+
       'enum explicit friend global integer let lib long module object option optional '+
       'preserve private property public redim single static string type variant'));

  VBStyles : Array [0..10] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'White space'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $007F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'Keyword'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Double quoted string'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $007F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Preprocessor'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Operators'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Identifier'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'Date'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $007F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 34; Name : 'Brace highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 35; Name : 'Brace Bad highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
     );

  CPPKeywords : Array [0..2] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords : 'asm auto bool break case catch char class const const_cast continue '+
       'default delete do double dynamic_cast else enum explicit export extern false float for '+
       'friend goto if inline int long mutable namespace new operator private protected public '+
       'register reinterpret_cast return short signed sizeof static static_cast struct switch '+
       'template this throw true try typedef typeid typename union unsigned using '+
       'virtual void volatile wchar_t while'),
      (Name : 'User-defined keywords';
       KeywordListNumber : 1;
       Keywords : ''),
      (Name : 'Doc Comments';
       KeywordListNumber : 2;
       Keywords : 'a addindex addtogroup anchor arg attention '+
       'author b brief bug c class code date def defgroup deprecated dontinclude '+
       'e em endcode endhtmlonly endif endlatexonly endlink endverbatim enum example exception '+
       'f$ f[ f] file fn hideinitializer htmlinclude htmlonly '+
       'if image include ingroup internal invariant interface latexonly li line link '+
       'mainpage name namespace nosubgrouping note overload '+
       'p page par param post pre ref relates remarks return retval '+
       'sa section see showinitializer since skip skipline struct subsection '+
       'test throw todo typedef union until '+
       'var verbatim verbinclude version warning weakgroup $ @ \ & < > # { }')
      );

  CPPStyles : Array [0..20] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'White space'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Line Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'Doc Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $3F703F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Double quoted string'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Single quoted string'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'UUIDs (only in IDL)'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $804080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 9; Name : 'Preprocessor'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $007F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 10; Name : 'Operators'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 11; Name : 'Identifier'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 12; Name : 'EOL if string is not closed'; FontName : 'Courier New'; FontSize : 0; FontStyles : [];
      ForeColor : $000000; BackColor : $E0C0E0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 13; Name : 'Verbatim strings for C#'; FontName : 'Courier New'; FontSize : 0; FontStyles : [];
      ForeColor : $007F00; BackColor : $E0FFE0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 14; Name : 'Regular expressions'; FontName : 'Courier New'; FontSize : 0; FontStyles : [];
      ForeColor : $3F7F3F; BackColor : $E0F0FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 15; Name : 'Doc Comment Line'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 16; Name : 'User-defined keywords'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $B00040; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 17; Name : 'Comment keyword'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $3060A0; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
      (StyleNumber : 18; Name : 'Comment keyword error'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $804020; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 34; Name : 'Brace highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 35; Name : 'Brace Bad highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
     );

  JavaScriptKeywords : Array [0..0] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords : 'abstract boolean break byte case catch char class  '+
       'const continue debugger default delete do double else enum export extends  '+
       'final finally float for function goto if implements import in instanceof  '+
       'int interface long native new package private protected public  '+
       'return short static super switch synchronized this throw throws  '+
       'transient try typeof var void volatile while with'));

  JavaKeywords : Array [0..0] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords : 'abstract boolean break byte case catch char class  '+
       'const continue debugger default delete do double else enum export extends  '+
       'final finally float for function goto if implements import in instanceof  '+
       'int interface long native new package private protected public  '+
       'return short static super switch synchronized this throw throws  '+
       'transient try typeof var void volatile while with'));

  HTMLKeywords : Array [0..5] of TSciLangKeywords =
     ((Name : 'HyperText';
       KeywordListNumber : 0;
       Keywords : 'a abbr acronym address applet area b base basefont '+
        'bdo big blockquote body br button caption center '+
        'cite code col colgroup dd del dfn dir div dl dt em '+
        'fieldset font form frame frameset h1 h2 h3 h4 h5 h6 '+
        'head hr html i iframe img input ins isindex kbd label '+
        'legend li link map menu meta noframes noscript '+
        'object ol optgroup option p param pre q s samp '+
        'script select small span strike strong style sub sup '+
        'table tbody td textarea tfoot th thead title tr tt u ul '+
        'var xml xmlns '+
        'abbr accept-charset accept accesskey action align alink '+
        'alt archive axis background bgcolor border '+
        'cellpadding cellspacing char charoff charset checked cite '+
        'class classid clear codebase codetype color cols colspan '+
        'compact content coords '+
        'data datafld dataformatas datapagesize datasrc datetime '+
        'declare defer dir disabled enctype event '+
        'face for frame frameborder '+
        'headers height href hreflang hspace http-equiv '+
        'id ismap label lang language leftmargin link longdesc '+
        'marginwidth marginheight maxlength media method multiple '+
        'name nohref noresize noshade nowrap '+
        'object onblur onchange onclick ondblclick onfocus '+
        'onkeydown onkeypress onkeyup onload onmousedown '+
        'onmousemove onmouseover onmouseout onmouseup '+
        'onreset onselect onsubmit onunload '+
        'profile prompt readonly rel rev rows rowspan rules '+
        'scheme scope selected shape size span src standby start style '+
        'summary tabindex target text title topmargin type usemap '+
        'valign value valuetype version vlink vspace width '+
        'text password checkbox radio submit reset '+
        'file hidden image '+
        'public !doctype'),
      (Name : 'JavaScript';
       KeywordListNumber : 1;
       Keywords : 'abstract boolean break byte case catch char class '+
        'const continue debugger default delete do double else enum export extends '+
        'final finally float for function goto if implements import in instanceof '+
        'int interface long native new package private protected public '+
        'return short static super switch synchronized this throw throws '+
        'transient try typeof var void volatile while with'),
      (Name : 'VBScript';
       KeywordListNumber : 2;
       Keywords : 'and begin case call class continue do each else elseif end erase '+
       'error event exit false for function get gosub goto if implement in load loop lset '+
       'me mid new next not nothing on or property raiseevent rem resume return rset '+
       'select set stop sub then to true unload until wend while with withevents '+
       'attribute alias as boolean byref byte byval const compare currency date declare dim double '+
       'enum explicit friend global integer let lib long module object option optional '+
       'preserve private property public redim single static string type variant'),
      (Name : 'Python';
       KeywordListNumber : 3;
       Keywords : 'and assert break class continue def del elif '+
       'else except exec finally for from global if import in is lambda None '+
       'not or pass print raise return try while yield'),
      (Name : 'PHP';
       KeywordListNumber : 4;
       Keywords : 'and argv as argc break case cfunction class continue declare default do '+
       'die echo else elseif empty enddeclare endfor endforeach endif endswitch '+
       'endwhile e_all e_parse e_error e_warning eval exit extends false for '+
       'foreach function global http_cookie_vars http_get_vars http_post_vars '+
       'http_post_files http_env_vars http_server_vars if include include_once '+
       'list new not null old_function or parent php_os php_self php_version '+
       'print require require_once return static switch stdclass this true var '+
       'xor virtual while __file__ __line__ __sleep __wakeup'),
      (Name : 'DTD Keywords';
       KeywordListNumber : 5;
       Keywords : 'ELEMENT DOCTYPE ATTLIST ENTITY NOTATION'));

  HTMLStyles : Array [0..94] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'Text'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Tags'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Unknown Tags'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'Attributes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Unknown Attributes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Numbers'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Double quoted strings'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Single quoted strings'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'Other inside tag'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 9; Name : 'Comment'; FontName : 'Verdana'; FontSize : 9; FontStyles : [];
      ForeColor : $008080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 10; Name : 'Entities'; FontName : 'Times New Roman'; FontSize : 11; FontStyles : [];
      ForeColor : $800080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 11; Name : 'XML style tag ends'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 12; Name : 'XML identifier start'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 13; Name : 'XML identifier end'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 14; Name : 'SCRIPT'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 15; Name : 'ASP <% ... %>'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $00FFFF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 16; Name : 'ASP <% ... %>'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $00DFFF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 17; Name : 'CDATA'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $00DFFF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 18; Name : 'PHP'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $FF0000; BackColor : $BFEFFF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 19; Name : 'Unquoted values'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $FF00FF; BackColor : $FFEFFF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 21; Name : 'SGML tags <! ... >'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 22; Name : 'SGML command'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $800000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 23; Name : 'SGML 1st param'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $006600; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 24; Name : 'SGML double string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000080; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 25; Name : 'SGML single string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $003399; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 26; Name : 'SGML error'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000080; BackColor : $6666FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 27; Name : 'SGML special'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $FF6633; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 28; Name : 'SGML entity'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $333333; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 29; Name : 'SGML comment'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $008080; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 31; Name : 'SGML block'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $660000; BackColor : $E0CCCC; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 34; Name : 'Matched Operators'; FontName : 'Times New Roman'; FontSize : 11; FontStyles : [];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 35; Name : 'Matched Operators'; FontName : 'Times New Roman'; FontSize : 11; FontStyles : [];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
      //# Embedded Javascript
     (StyleNumber : 40; Name : 'JS Start'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $007F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 41; Name : 'JS Default'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [fsBold];
      ForeColor : $000000; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 42; Name : 'JS Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 43; Name : 'JS Line Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 44; Name : 'JS Doc Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [fsBold];
      ForeColor : $3F703F; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 45; Name : 'JS Number'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $7F7F00; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 46; Name : 'JS Word'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $000000; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 47; Name : 'JS Keyword'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [fsBold];
      ForeColor : $7F0000; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 48; Name : 'JS Double quoted string'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [fsBold];
      ForeColor : $7F007F; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 49; Name : 'JS Single quoted string'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [fsBold];
      ForeColor : $7F007F; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 50; Name : 'JS Symbols'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [fsBold];
      ForeColor : $000000; BackColor : $FFF0F0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 51; Name : 'JS EOL'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $B0BBBF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 52; Name : 'JS 52'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $B0BBFF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
      //# Embedded VBS
     (StyleNumber : 71; Name : 'VBS Default'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $000000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 72; Name : 'VBS Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $008000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 73; Name : 'VBS Number'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $808000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 74; Name : 'VBS KeyWord'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [fsBold];
      ForeColor : $800000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 75; Name : 'VBS String'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $800080; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 76; Name : 'VBS Identifier'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $800000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 77; Name : 'VBS Unterminated string'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $800000; BackColor : $FF7F7F; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
      //# ASP VBS
     (StyleNumber : 81; Name : 'ASP Default'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $000000; BackColor : $CFCFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 82; Name : 'ASP Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $008000; BackColor : $CFCFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 83; Name : 'ASP Number'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $808000; BackColor : $CFCFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 84; Name : 'ASP KeyWord'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [fsBold];
      ForeColor : $800000; BackColor : $CFCFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 85; Name : 'ASP String'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $800080; BackColor : $CFCFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 86; Name : 'ASP Identifier'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $800000; BackColor : $CFCFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 87; Name : 'ASP Unterminated string'; FontName : 'Lucida Sans Unicode'; FontSize : 10; FontStyles : [];
      ForeColor : $800000; BackColor : $BF7F7F; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
      //# Embedded Python
     (StyleNumber : 90; Name : 'Python'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 91; Name : 'Python'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 92; Name : 'Python Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 93; Name : 'Python Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 94; Name : 'Python String'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 95; Name : 'Python Single quoted string'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 96; Name : 'Python Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F0000; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 97; Name : 'Python Triple quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 98; Name : 'Python Triple double quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 99; Name : 'Python Class name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 100; Name : 'Python function or method name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F7F00; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 101; Name : 'Python function or method name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : clDefault; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 102; Name : 'Python Identifiers'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
      //# ASP Python
     (StyleNumber : 105; Name : 'ASP Python'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 106; Name : 'ASP Python'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 107; Name : 'ASP Python Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 108; Name : 'ASP Python Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 109; Name : 'ASP Python String'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 110; Name : 'ASP Python Single quoted string'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 111; Name : 'ASP Python Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F0000; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 112; Name : 'ASP Python Triple quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 113; Name : 'ASP Python Triple double quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 114; Name : 'ASP Python Class name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 115; Name : 'ASP Python function or method name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F7F00; BackColor : $EFFFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 116; Name : 'ASP Python function or method name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : clDefault; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 117; Name : 'ASP Python Identifiers'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : $CFEFCF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
      //# PHP
     (StyleNumber : 118; Name : 'PHP Default'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $330000; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 119; Name : 'PHP Double quoted string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $007F00; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 120; Name : 'PHP Single quoted string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $009F00; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 121; Name : 'PHP Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F007F; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 122; Name : 'PHP Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $0099CC; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 123; Name : 'PHP Variable'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F0000; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 124; Name : 'PHP Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $999999; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 125; Name : 'PHP One line Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $666666; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 126; Name : 'PHP Variable in double quoted string'; FontName : ''; FontSize : 0; FontStyles : [fsItalic];
      ForeColor : $7F0000; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 127; Name : 'PHP operator'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000000; BackColor : $F8F8FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
    );

  XMLStyles : Array [0..26] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'Default'; FontName : 'Times New Roman'; FontSize : 11; FontStyles : [];
      ForeColor : $000000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Tags'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Unknown Tags'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'Attributes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Unknown Attributes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Numbers'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Double quoted strings'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Single quoted strings'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'Other inside tag'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 9; Name : 'Comment'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $008080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 10; Name : 'Entities'; FontName : 'Times New Roman'; FontSize : 11; FontStyles : [];
      ForeColor : $800080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 11; Name : 'XML style tag ends'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 12; Name : 'XML identifier start'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $800080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 13; Name : 'XML identifier end'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $800080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 17; Name : 'CDATA'; FontName : 'Times New Roman'; FontSize : 11; FontStyles : [];
      ForeColor : $000080; BackColor : $F0F0FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 18; Name : 'XML Question'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 19; Name : 'Unquoted values'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $608060; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 21; Name : 'SGML tags <! ... >'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $800000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 22; Name : 'SGML command'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $800000; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 23; Name : 'SGML 1st param'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $006600; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 24; Name : 'SGML double string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000080; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 25; Name : 'SGML single string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $003399; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 26; Name : 'SGML error'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $000080; BackColor : $6666FF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 27; Name : 'SGML special'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $FF6633; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 28; Name : 'SGML entity'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $333333; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 29; Name : 'SGML comment'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $008080; BackColor : $FFEFEF; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 31; Name : 'SGML block'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $660000; BackColor : $E0CCCC; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
    );

  PythonKeywords : Array [0..0] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords : 'and assert break class continue def del elif ' +
       'else except exec finally for from global if import in is lambda None ' +
       'not or pass print raise return try while yield'));

  PythonStyles : Array [0..15] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'White space'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'String'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Single quoted string'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F00000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Triple quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Triple quoted string'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'Class name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 9; Name : 'Function or method name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 10; Name : 'Operators'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : clDefault; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 11; Name : 'Identifiers'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : clDefault; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 12; Name : 'Comment blocks'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 13; Name : 'End of line where string is not closed'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $000000; BackColor : $E0C0E0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 34; Name : 'Brace highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 35; Name : 'Brace Bad highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
     );

  RubyKeywords : Array [0..0] of TSciLangKeywords =
     ((Name : 'Keywords';
       KeywordListNumber : 0;
       Keywords : '__FILE__ and def end in or self unless __LINE__ begin ' +
       'defined? ensure module redo super until BEGIN break do false next rescue ' +
       'then when END case else for nil require retry true while alias class elsif if ' +
       'not return undef yield'));

  RubyStyles : Array [0..15] of TSciLangStyle =
    ((StyleNumber : 0; Name : 'White space'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $808080; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 1; Name : 'Comment'; FontName : 'Comic Sans MS'; FontSize : 9; FontStyles : [];
      ForeColor : $007F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 2; Name : 'Number'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 3; Name : 'String'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 4; Name : 'Single quoted string'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $7F007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 5; Name : 'Keyword'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F00000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 6; Name : 'Triple quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 7; Name : 'Triple double quotes'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $00007F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 8; Name : 'Class name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 9; Name : 'Function or method name definition'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $7F7F00; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 10; Name : 'Operators'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : clDefault; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 11; Name : 'Identifiers'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 12; Name : 'Comment blocks'; FontName : ''; FontSize : 0; FontStyles : [];
      ForeColor : $7F7F7F; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 13; Name : 'End of line where string is not closed'; FontName : 'Courier New'; FontSize : 10; FontStyles : [];
      ForeColor : $000000; BackColor : $E0C0E0; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : True),
     (StyleNumber : 34; Name : 'Brace highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $FF0000; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False),
     (StyleNumber : 35; Name : 'Brace Bad highlight'; FontName : ''; FontSize : 0; FontStyles : [fsBold];
      ForeColor : $0000FF; BackColor : clDefault; CharCase : CASE_MIXED; Visible: True;
      Changeable : True; HotSpot : False;  EOLFilled : False)
     );

implementation

Uses
  SciLexer;

{ TSciKeywords }

procedure TSciKeywords.Assign(Source: TPersistent);
begin
  if Source is TSciKeywords then begin
    fName := TSciKeywords(Source).Name;
    fKeyWordListNumber := TSciKeywords(Source).KeywordListNumber;
    fKeywords.Assign(TSciKeywords(Source).Keywords);
  end else
    inherited;
end;

procedure TSciKeywords.AssignRec(Rec : TSciLangKeywords);
begin
  fName := Rec.Name;
  fKeywordListNumber := Rec.KeywordListNumber;
  fKeywords.Delimiter := ' ';
  fKeywords.DelimitedText := Rec.KeyWords;
end;

constructor TSciKeywords.Create(Collection: TCollection);
begin
  fKeywords := TStringList.Create;
  inherited;
  fName := 'Keywords['+IntToStr(Index)+']';
end;

destructor TSciKeywords.Destroy;
begin
  fKeywords.Free;
  inherited;
end;

procedure TSciKeywords.SetKeywordListNumber(const Value: integer);
begin
  FKeywordListNumber := Value;
  Changed(False);
end;

procedure TSciKeywords.SetKeywords(const Value: TStrings);
begin
  fKeyWords.Assign(Value);
  Changed(False);
end;

procedure TSciKeywords.SetName(const Value: string);
begin
  fName := Value;
  Changed(False);
end;

{ TSciStyle }

procedure TSciStyle.Assign(Source: TPersistent);
begin
  if Source is TSciStyle then begin
  fStyleNumber := TSciStyle(Source).StyleNumber;
  fName := TSciStyle(Source).Name;
  fFontName := TSciStyle(Source).FontName;
  fFontSize := TSciStyle(Source).FontSize;
  fFontStyles := TSciStyle(Source).FontStyles;
  fForeColor :=TSciStyle(Source).ForeColor;
  fBackColor := TSciStyle(Source).BackColor;
  fCharCase := TSciStyle(Source).CharCase;
  fVisible := TSciStyle(Source).Visible;
  fChangeable := TSciStyle(Source).Changeable;
  fHotspot := TSciStyle(Source).HotSpot;
  fEOLFilled := TSciStyle(Source).EOLFilled;
  end else
    inherited;
end;

procedure TSciStyle.AssignRec(Rec: TSciLangStyle);
begin
  fStyleNumber := Rec.StyleNumber;
  fName := Rec.Name;
  fFontName := Rec.FontName;
  fFontSize := Rec.FontSize;
  fFontStyles := Rec.FontStyles;
  fForeColor :=Rec.ForeColor;
  fBackColor := Rec.BackColor;
  fCharCase := Rec.CharCase;
  fVisible := Rec.Visible;
  fChangeable := Rec.Changeable;
  fHotspot := Rec.Hotspot;
  fEOLFilled := Rec.EOLFilled;
end;

constructor TSciStyle.Create(Collection: TCollection);
begin
  inherited;
  fStyleNumber := Index;
  fName := 'Style['+IntToStr(Index)+']';
  fFontName := '';
  fFontSize := 0;
  fForeColor := clDefault;
  fBackColor := clDefault;
  fVisible := True;
  fEOLFilled := False;
  fChangeable := True;
  fHotSpot := False;
end;

procedure TSciStyle.SetBackColor(const Value: TColor);
begin
  fBackColor := Value;
  Changed(False);
end;

procedure TSciStyle.SetChangeable(const Value: Boolean);
begin
  fChangeable := Value;
  Changed(False);
end;

procedure TSciStyle.SetCharCase(const Value: TSciCase);
begin
  fCharCase := Value;
  Changed(False);
end;

procedure TSciStyle.SetEOLFilled(const Value: Boolean);
begin
  fEOLFilled := Value;
  Changed(False);
end;

procedure TSciStyle.SetFontName(const Value: string);
begin
  fFontName := Value;
  Changed(False);
end;

procedure TSciStyle.SetFontSize(const Value: integer);
begin
  fFontSize := Value;
  Changed(False);
end;

procedure TSciStyle.SetFontStyles(const Value: TFontStyles);
begin
  FFontStyles := Value;
  Changed(False);
end;

procedure TSciStyle.SetForeColor(const Value: TColor);
begin
  fForeColor := Value;
  Changed(False);
end;

procedure TSciStyle.SetHotspot(const Value: Boolean);
begin
  fHotSpot := Value;
  Changed(False);
end;

procedure TSciStyle.SetName(const Value: string);
begin
  fName := Value;
  Changed(False);
end;

procedure TSciStyle.SetStyleNumber(const Value: integer);
begin
  fStyleNumber := Value;
  Changed(False);
end;

procedure TSciStyle.SetVisible(const Value: Boolean);
begin
  fVisible := Value;
  Changed(False);
end;

{ TSciKeyWordsList }

procedure TSciKeyWordsList.SetEditor(Editor: TPersistent);
begin
  fEditor := Editor;
end;

procedure TSciKeyWordsList.Update(Item: TCollectionItem);
Var
  i : integer;
begin
  inherited;
  if not Assigned(fEditor) then Exit;
  if Assigned(Item) then with (Item as TSciKeywords) do begin
    Keywords.Delimiter := ' ';
    TScintilla(fEditor).SetKeywords(KeywordListNumber, PChar(Keywords.DelimitedText));
  end else begin
    for i := 0 to Count - 1 do with (Items[i] as TSciKeywords) do begin
      Keywords.Delimiter := ' ';
      TScintilla(fEditor).SetKeywords(KeywordListNumber, PChar(Keywords.DelimitedText));
    end;
  end;
end;

{ TSciStyleList }

procedure TSciStyleList.SetEditor(Editor: TPersistent);
begin
  fEditor := Editor;
end;

procedure TSciStyleList.Update(Item: TCollectionItem);
Var
  i : integer;
begin
  inherited;
  if not Assigned(fEditor) then Exit;
  if Assigned(Item) then with (Item as TSciStyle), TScintilla(fEditor) do begin
    if fFontName <> '' then StyleSetFont(StyleNumber, PChar(fFontName));
    if fFontSize > 0 then StyleSetSize (StyleNumber, fFontSize);
    if fForeColor <> clDefault then StyleSetFore(StyleNumber, ColorToRGB(fForeColor));
    if fBackColor <> clDefault then StyleSetBack(StyleNumber, ColorToRGB(fBackColor));
    StyleSetBold(StyleNumber, fsBold in fFontStyles);
    StyleSetItalic(StyleNumber, fsItalic in fFontStyles);
    StyleSetUnderline(StyleNumber, fsUnderline in fFontStyles);
    StyleSetCase(StyleNumber, Ord(fCharCase));
    StyleSetVisible(StyleNumber, fVisible);
    StyleSetChangeable(StyleNumber, fChangeable);
    StyleSetHotSpot(StyleNumber, fHotSpot);
    StyleSetEOLFilled(StyleNumber, fEOLFilled);
  end else
    for i := 0 to Count - 1 do Update(Items[i]);
end;

{ TSciSynLexer }

procedure TSciSynLexer.Assign(Source: TPersistent);
begin
  if Source is TSciSynLexer then begin
    fLanguage := TSciSynLexer(Source).Language;
    fKeywordsList.Assign(TSciSynLexer(Source).KeywordsList);
    fStyleList.Assign(TSciSynLexer(Source).StyleList);
    Update;
  end else
    inherited;
end;

constructor TSciSynLexer.Create(Editor: TPersistent);
begin
  inherited Create;
  fEditor := Editor;
  fLanguage := Syn_NULL;
  fKeywordsList := TSciKeyWordsList.Create(TSciKeywords);
  fKeywordsList.SetEditor(Editor);
  fStyleList := TSciStyleList.Create(TSciStyle);
  fStyleList.SetEditor(Editor);
end;

destructor TSciSynLexer.Destroy;
begin
  fKeywordsList.Free;
  fStyleList.Free;
  inherited;
end;

function TSciSynLexer.GetOwner: TPersistent;
begin
  Result := fEditor;
end;

procedure TSciSynLexer.SetKeywords(Keywords: array of TSciLangKeywords);
var
  i : integer;
begin
  fKeywordsList.BeginUpdate;
  try
    fKeywordsList.Clear;
    for i := Low(Keywords) to High(Keywords) do
      TSciKeywords(fKeywordsList.Add).AssignRec(Keywords[i]);
  finally
    fKeywordsList.EndUpdate;
  end;
  Update;
end;

procedure TSciSynLexer.SetKeywordsList(const Value: TSciKeyWordsList);
begin
  fKeywordsList.Assign(Value);
  fKeywordsList.Update(nil);
end;

procedure TSciSynLexer.SetLanguage(const Value: TSCLEX);

  procedure AddKeyWords(KeywordRecs : array of TSciLangKeywords);
  var
    i : integer;
  begin
    for i := Low(KeywordRecs) to High(KeywordRecs) do
      TSciKeywords(fKeywordsList.Add).AssignRec(KeywordRecs[i]);
  end;

  procedure AddStyles(Styles : array of TSciLangStyle);
  var
    i : integer;
  begin
    for i := Low(Styles) to High(Styles) do
      TSciStyle(fStyleList.Add).AssignRec(Styles[i]);
  end;

begin
  fLanguage := Value;
  { Here we need to customise different Language Lexers }
  fKeywordsList.BeginUpdate;
  fStyleList.BeginUpdate;
  try
    fStyleList.Clear;
    fKeywordsList.Clear;
    case fLanguage of
      Syn_CPP :
        begin
          AddKeywords(CPPKeywords);
          AddStyles(CPPStyles);
        end;
      Syn_PASCAL :
        begin
          AddKeywords(PascalKeywords);
          AddStyles(PascalStyles);
        end;
      Syn_VB, Syn_VBSCRIPT :
        begin
          AddKeywords(VBKeywords);
          AddStyles(VBStyles);
        end;
      Syn_Python :
        begin
          AddKeywords(PythonKeywords);
          AddStyles(PythonStyles);
        end;
      Syn_Ruby :
        begin
          AddKeywords(RubyKeywords);
          AddStyles(RubyStyles);
        end;
      Syn_HTML :
        begin
          AddKeywords(HTMLKeyWords);
          AddStyles(HTMLStyles);
          if Assigned(FEditor) then
            TScintilla(FEditor).SetProperty('fold.html', '1');
        end;
      Syn_XML :
        begin
          AddStyles(XMLStyles);
          TScintilla(FEditor).SetProperty('fold.html', '1');
        end;
    end;
  finally
    fKeywordsList.EndUpdate;
    fStyleList.EndUpdate;
  end;
  Update;
end;

procedure TSciSynLexer.SetStyleList(const Value: TSciStyleList);
begin
  fStyleList.Assign(Value);
  fStyleList.Update(nil);
end;

procedure TSciSynLexer.SetStyles(Styles: array of TSciLangStyle);
var
  i : integer;
begin
  fStyleList.BeginUpdate;
  try
    fStyleList.Clear;
    for i := Low(Styles) to High(Styles) do
      TSciStyle(fStyleList.Add).AssignRec(Styles[i]);
  finally
    fStyleList.EndUpdate;
  end;
  Update;
end;

procedure TSciSynLexer.Update;
begin
  inherited;
  if not Assigned(fEditor) then Exit;
  with TScintilla(fEditor) do begin
    SetLexer(Ord(fLanguage));
    StyleClearAll;
    if fLanguage = Syn_HTML then
      SetStyleBits(7)
    else
      SetStyleBits(5);
    fKeywordsList.Update(nil);
    fStyleList.Update(nil);
  end;
end;

end.
